
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.util.Random;
import java.util.ArrayList;
import java.io.*;


/**
 * This class implements a game playing client which makes random
 * but valid moves for the CS3243 (2010 Fall) HW1 Ataxx game.
 *
 * Game board informatiom is received from stdin, while 
 * moves this class makes are output via stdout.
 *
 * The quickest way to get a working game client is just to customise 
 * 1. the constructor RandomPlayer()
 * 2. the function MakeMove()
 *
 * @author NG, Jun Ping, ngjp@nus.edu.sg
 *
 */
public class RandomPlayer {

   
    // Used to hold the name we want to identify ourselves by
    private String m_ClientName;

    // These variables are used to track information about the game environment
    private int m_TimeOutInSeconds; // How many seconds we have per move
    private int m_PlayerID;         // In this assignment, we assume we are always 1!

    // A random number generator
    private Random m_RNG;
    
    // These variables deal with the I/O
    private BufferedReader m_In;
    private BufferedWriter m_Out;


    /**
     * Constructor.
     * All initialisation should be done here.
     * @param a_ClientIndex [in] the ID that the server has assigned to us.
     */
    public RandomPlayer() {
    
        
        // More customisations here if necessary
        // ===============
        m_ClientName = "The Four-Leaf Clover"; // <-- Change this!
        m_TimeOutInSeconds = 3; // <-- This is just for your reference. The server will manage the timeouts!
        m_PlayerID = 1;         // We assume we are player 1. Change this, and likely you won't do very well!
        
        // A random generator - for the RandomPlayer
        // You probably don't need this in your own code!
        m_RNG = new Random();
        
        
        // Set up buffers to make it easier to deal with stream I/O
        // You don't need to change this likely!
        try {
            m_In = new BufferedReader(new InputStreamReader(System.in));
            m_Out = new BufferedWriter(new OutputStreamWriter(System.out));
        } catch (Exception e) {
            // Honestly, if we have an exception, we cannot recover.
            // All the best!
            //System.err.println("Unable to set up buffers for I/O.");
        }
    
    
    } // end Constructor
    
    
    /**
     * This function decides a move to make next.
     * We are only making random moves here.
     * And the function only moves to immediately adjacent tiles even though
     * we can make jumps.
     *
     * You should change this to do something more intelligent.
     * The return value should be a 2-D array with the format
     * [0][0] X-OLD
     * [0][1] Y-OLD
     * [1][0] X-NEW
     * [1][1] Y-NEW
     * where X-OLD is the x coordinate of the tile to move from
     *       Y-OLD is the y coordinate of the tile to move from
     *       X-NEW is the x coordinate of the tile to move to
     *       Y-NEW is the y coordinate of the tile to move to
     *
     * If your move is invalid, your turn will be forfeited.
     *
     * @return array containing old move, and the new move to take.
     */
    private int[][] MakeMove(GameBoard a_GameBoard) {
    
        int[][] ReturnMove = new int[2][2];
        
        // 1. Determine valid locations we can make a move from
        ArrayList<int[]> ValidMoves = GetPossibleStartLocations(a_GameBoard);        
        if (ValidMoves.size() == 0) {
            // No moves possible, we can only forfeit the move
            // Tough luck!
            return null;
        }
        int RandomStartLocation = m_RNG.nextInt(ValidMoves.size());        
        int[] StartTile = ValidMoves.get(RandomStartLocation);
        int[] EndTile = new int[2];
        
        // Try to make a move, any move that's valid from here
        // 1. Try horizontal
        try {
            if (a_GameBoard.GetValueAtTile(StartTile[0]+1, StartTile[1]) == 0) {
                // Make move!
                EndTile[0] = StartTile[0]+1;
                EndTile[1] = StartTile[1];
                ReturnMove[0][0] = StartTile[0];
                ReturnMove[0][1] = StartTile[1];
                ReturnMove[1][0] = EndTile[0];
                ReturnMove[1][1] = EndTile[1];
                return ReturnMove;
            }           
        } catch (Exception e) {
            // Invalid tile position            
        }
        try {
            if (a_GameBoard.GetValueAtTile(StartTile[0]-1, StartTile[1]) == 0) {
                // Make move!
                EndTile[0] = StartTile[0]-1;
                EndTile[1] = StartTile[1];
                ReturnMove[0][0] = StartTile[0];
                ReturnMove[0][1] = StartTile[1];
                ReturnMove[1][0] = EndTile[0];
                ReturnMove[1][1] = EndTile[1];
                return ReturnMove;
            }           
        } catch (Exception e) {
            // Invalid tile position            
        }
        
        // 2. Try vertical
        try {
            if (a_GameBoard.GetValueAtTile(StartTile[0], StartTile[1]+1) == 0) {
                // Make move!
                EndTile[0] = StartTile[0];
                EndTile[1] = StartTile[1]+1;
                ReturnMove[0][0] = StartTile[0];
                ReturnMove[0][1] = StartTile[1];
                ReturnMove[1][0] = EndTile[0];
                ReturnMove[1][1] = EndTile[1];
                return ReturnMove;
            }           
        } catch (Exception e) {
            // Invalid tile position            
        }
        try {
            if (a_GameBoard.GetValueAtTile(StartTile[0], StartTile[1]-1) == 0) {
                // Make move!
                EndTile[0] = StartTile[0];
                EndTile[1] = StartTile[1]-1;
                ReturnMove[0][0] = StartTile[0];
                ReturnMove[0][1] = StartTile[1];
                ReturnMove[1][0] = EndTile[0];
                ReturnMove[1][1] = EndTile[1];
                return ReturnMove;
            }           
        } catch (Exception e) {
            // Invalid tile position            
        }
        
        // 3. Try diagonal
        try {
            if (a_GameBoard.GetValueAtTile(StartTile[0]+1, StartTile[1]+1) == 0) {
                // Make move!
                EndTile[0] = StartTile[0]+1;
                EndTile[1] = StartTile[1]+1;
                ReturnMove[0][0] = StartTile[0];
                ReturnMove[0][1] = StartTile[1];
                ReturnMove[1][0] = EndTile[0];
                ReturnMove[1][1] = EndTile[1];
                return ReturnMove;
            }           
        } catch (Exception e) {
            // Invalid tile position            
        }
        try {
            if (a_GameBoard.GetValueAtTile(StartTile[0]+1, StartTile[1]-1) == 0) {
                // Make move!
                EndTile[0] = StartTile[0]+1;
                EndTile[1] = StartTile[1]-1;
                ReturnMove[0][0] = StartTile[0];
                ReturnMove[0][1] = StartTile[1];
                ReturnMove[1][0] = EndTile[0];
                ReturnMove[1][1] = EndTile[1];
                return ReturnMove;
            }           
        } catch (Exception e) {
            // Invalid tile position            
        }
        try {
            if (a_GameBoard.GetValueAtTile(StartTile[0]-1, StartTile[1]+1) == 0) {
                // Make move!
                EndTile[0] = StartTile[0]-1;
                EndTile[1] = StartTile[1]+1;
                ReturnMove[0][0] = StartTile[0];
                ReturnMove[0][1] = StartTile[1];
                ReturnMove[1][0] = EndTile[0];
                ReturnMove[1][1] = EndTile[1];
                return ReturnMove;
            }           
        } catch (Exception e) {
            // Invalid tile position            
        }
        try {
            if (a_GameBoard.GetValueAtTile(StartTile[0]-1, StartTile[1]-1) == 0) {
                // Make move!
                EndTile[0] = StartTile[0]-1;
                EndTile[1] = StartTile[1]-1;
                ReturnMove[0][0] = StartTile[0];
                ReturnMove[0][1] = StartTile[1];
                ReturnMove[1][0] = EndTile[0];
                ReturnMove[1][1] = EndTile[1];
                return ReturnMove;
            }           
        } catch (Exception e) {
            // Invalid tile position            
        }

        
        
        return ReturnMove;
    
    } // end MakeMove()
    
    
    /**
     * This function goes through the game board, and returns a set of possible
     * start locations from which we can make a move from.
     * A possible start location will be tiles which we already occupy, and
     * which have adjacent space for us to move into.
     * @param a_GameBoard [in] the current game board
     * @return an array list of possible locations, represented as an 2-element
     *         array.
     *
     */
    private ArrayList<int[]> GetPossibleStartLocations(GameBoard a_GameBoard) {
    
        ArrayList<int[]> l_ReturnList = new ArrayList<int[]>();
        
        // Go through every tile
        for (int i = 0; i < a_GameBoard.GetBoardLength(); ++i) {
            for (int j = 0; j < a_GameBoard.GetBoardLength(); ++j) {
            
                // Check if this is our tile
                try {
                    if (a_GameBoard.GetValueAtTile(i,j) != m_PlayerID) {
                        // Not our tile, we skip!
                        continue;
                    }
                } catch (Exception e) {
                    // Unable to access tile, we skip too!
                    continue;
                }
            
                // Check if this is a possible start location
                // 1. Horizontal tiles?
                try {
                    if (a_GameBoard.GetValueAtTile(i+1,j) == 0 || a_GameBoard.GetValueAtTile(i-1, j) == 0) {
                        int[] l_T = new int[2];
                        l_T[0] = i;
                        l_T[1] = j;
                        l_ReturnList.add(l_T);
                        continue;
                    }
                } catch (Exception e) {
                }
                // 2. vertical tiles?
                try {
                    if (a_GameBoard.GetValueAtTile(i,j+1) == 0 || a_GameBoard.GetValueAtTile(i, j-1) == 0) {
                        int[] l_T = new int[2];
                        l_T[0] = i;
                        l_T[1] = j;
                        l_ReturnList.add(l_T);
                        continue;                        
                    }
                } catch (Exception e) {
                }
                // 3. Diagonal tiles?
                try {
                    if (a_GameBoard.GetValueAtTile(i+1,j+1) == 0 || a_GameBoard.GetValueAtTile(i+1, j-1) == 0 ||
                    a_GameBoard.GetValueAtTile(i-1, j+1) == 0 ||
                    a_GameBoard.GetValueAtTile(i-1, j-1) == 0) 
                    {
                        int[] l_T = new int[2];
                        l_T[0] = i;
                        l_T[1] = j;
                        l_ReturnList.add(l_T);
                    }
                } catch (Exception e) {
                }
            
            } // end for j
        } // end for i
                
        return l_ReturnList;
    
    } // end GetPossibleStartLocations()
    
    
    /**
     * This function is the main part of this class.
     * We will wait for a command from stdin.
     * It could be one of:
     * 1. identify
     * 2. move
     * For more details, please refer to the course web page.
     * After servicing the command, we will exit the function (and this program)
     *
     */
    public void GoOnline() {

        // Wait for command
        String l_Command = ReadLineFromStdin();
        
        if (l_Command.compareTo("identify") == 0) {
            
            // We are asked for our name
            SendLineToStdout(m_ClientName);
            
        } else if (l_Command.compareTo("move") == 0) {
            
            // We need to put on our thinking cap and decide on a move to make
            // 1. Read in details about the game environment first
            try {
                String l_CurrentRoundNumStr = ReadLineFromStdin();
                int l_CurrentRoundNum = new Integer(l_CurrentRoundNumStr).intValue();
                String l_LengthOfSquareBoardStr = ReadLineFromStdin();
                int l_LengthOfSquareBoard = new Integer(l_LengthOfSquareBoardStr).intValue();
                String l_GameBoardStr = "";
                for (int i = 0; i < l_LengthOfSquareBoard; ++i) {
                    l_GameBoardStr += ReadLineFromStdin();
                }
                
                GameBoard l_GameBoard = new GameBoard(l_LengthOfSquareBoard, l_GameBoardStr);
                int[][] l_MoveToMake = MakeMove(l_GameBoard);
                    
                // If we managed to get a move, we'd send it to the server            
                if (l_MoveToMake != null) {
                
                    // Send the move to the server
                    // We need to convert it to a string first    
                    String l_MoveToMakeStr = "";

                    l_MoveToMakeStr += 
			(new Integer(l_MoveToMake[0][0])).toString() + ":" +
			(new Integer(l_MoveToMake[0][1])).toString() + ":" +
			(new Integer(l_MoveToMake[1][0])).toString() + ":" +
			(new Integer(l_MoveToMake[1][1])).toString() + ":";

                    SendLineToStdout(l_MoveToMakeStr);
                    
                } else {
                    
                    // We can't find a move to make.
                    // Silently pass then...
                    //System.err.println("Unable to find a move to make");
                    
                }
                
                
            } catch (NumberFormatException e) {
                // Error receiving input from server.
                // Hard to recover from, so we quit.
            } catch (Exception e) {
                // Probably an error receiving inputs from the server.
                // We have to quit.
            }           
            
        } else {
            
            // Unknown command.            
            // We'd terminate then.
            
        } // end if-else...
        
        
        // That's all.
        
    } // end PlayGame()
    
    
    /**
     * This function retrieves a line from System.in.
     * The line is expected to be terminated by a \n.
     * However we do not return the \n from this function.
     *
     * @return retrieved line, or null on errors.
     */
    private String ReadLineFromStdin() {
    
        try {
        
            // Listen on stdin 
            String l_InLine = m_In.readLine();
            if (l_InLine == null) {
                // The input stream seems to have closed.
                return null;
            } else {
                return l_InLine;
            }
           
        } catch (IOException e) {
            return null;
        }
        
    
    } // end RecvFromServer()
    
    
    /**
     * This function takes an input String and sends it to the server.
     * We will terminate the string to send with a \n.     
     *
     * @param a_StringToSend [in] the String to transmit to the server
     *
     */
    private void SendLineToStdout(String a_StringToSend) {
    
        //System.err.println("Sending to server: " + a_StringToSend);        
        try {
            m_Out.write(a_StringToSend);
            m_Out.flush();
        } catch (IOException e) {
            // Error, oh well...
        }
    
    } // end SendToServer()
    
    
   

    /**
     * Program entry point
     **/    
    public static void main(String args[]) {
                                
        // We are not expecting any arguments
                
        // Start an instance of the player and start playing the game!
        RandomPlayer l_Player = new RandomPlayer();
        l_Player.GoOnline();
        
        
    } // end main()



    /** 
     * This internal class helps us manipulate and access the game board.
     * Provided just for convenience, feel free to use it, or not...
     */
    private class GameBoard {
    
        // Holds information about the game board
        private int m_BoardLength;
        private int m_GameBoard[][];
        
        /**
         * Constructor.
         * Initialises an instance of this game board object.
         * @param a_BoardLength [in] the length of the square game board.
         * @param a_BoardDetails [in] a string which captures the state of the game board.
         * @throws Exception if there are errors with the provided arguments.
         */
        public GameBoard(int a_BoardLength, String a_BoardDetails) throws Exception {
        
            //System.err.println("BoardLength: " + a_BoardLength);
            //System.err.println("BoardDetails: " + a_BoardDetails);            
        
            // Set up the data structures
            m_BoardLength = a_BoardLength;
            for (int i = 0; i < a_BoardLength; ++i) {
            
                m_GameBoard = new int[a_BoardLength][];
                
                for (int j = 0; j < a_BoardLength; ++j) {                
                    m_GameBoard[j] = new int[a_BoardLength];
                } // end for j
                
            } // end for i
            
            
            // Populate the board with what we know
            StringTokenizer st = new StringTokenizer(a_BoardDetails, ":");            
            for (int i = 0; i < a_BoardLength; ++i) {
                for (int j = 0; j < a_BoardLength; ++j) {
                    try {
                        String Value = st.nextToken();
                        // Note we do j,i and not i,j so that the first index can
                        // refer to the column, and the second index refers to the row
                        m_GameBoard[j][i] = (new Integer(Value)).intValue();
                    } catch (NoSuchElementException e) {                        
                        throw new Exception("Unable to parse received board information.");
                    } catch (NumberFormatException e) {                        
                        throw new Exception("Invalid tile information");
                    }
                } // end for j
            } // end for i
        
        } // end constructor
        
        
        /**
         * Retrieves the length of the board. This is a square board remember!
         * @return number of tiles on one side of the square board.
         */
        public int GetBoardLength() {
            return m_BoardLength;
        } // end GetBoardLength()
        
        
        /**
         * This function retrieves the current value at a particular tile on the game board.
         *
         * @param i [in] the x-coordinate
         * @param j [in] the y-coordinate
         * @return the value at the tile position specified.
         * @throws InvalidParameterException if the arguments are not valid.
         */
        public int GetValueAtTile(int i, int j) throws Exception {
        
            if ((i >= 0 && i < m_BoardLength) && (j >= 0 && j < m_BoardLength)) {
                return m_GameBoard[i][j];
            } else {
                throw new Exception("Input arguments not valid!");
            }
            
        } // end GetValueAtTile
    
    } // end GameBoard
    

} // end class

